@@ -6,7 +6,6 @@ import (
 	"bytes"
 	"cmp"
 	"encoding/csv"
-	"encoding/json"
 	"fmt"
 	"math"
 	"os"
@@ -17,6 +16,7 @@ import (
 	"strings"
 	"time"
 
+	jsoniter "github.com/json-iterator/go"
 	"github.com/mattn/go-runewidth"
 
 	glanguage "golang.org/x/text/language"
@@ -76,28 +76,28 @@ var openMetricsSummaryRecordFormat = "scc_%s{language=\"%s\"} %d\n"
 var openMetricsFileRecordFormat = "scc_%s{language=\"%s\",file=\"%s\"} %d\n"
 
 func sortSummaryFiles(summary *LanguageSummary) {
-	switch {
-	case SortBy == "name" || SortBy == "names" || SortBy == "language" || SortBy == "languages" || SortBy == "lang":
+	switch SortBy {
+	case "name", "names", "language", "languages", "lang", "langs":
 		slices.SortFunc(summary.Files, func(a, b *FileJob) int {
 			return strings.Compare(a.Location, b.Location)
 		})
-	case SortBy == "line" || SortBy == "lines":
+	case "line", "lines":
 		slices.SortFunc(summary.Files, func(a, b *FileJob) int {
 			return cmp.Compare(b.Lines, a.Lines)
 		})
-	case SortBy == "blank" || SortBy == "blanks":
+	case "blank", "blanks":
 		slices.SortFunc(summary.Files, func(a, b *FileJob) int {
 			return cmp.Compare(b.Blank, a.Blank)
 		})
-	case SortBy == "code" || SortBy == "codes":
+	case "code", "codes":
 		slices.SortFunc(summary.Files, func(a, b *FileJob) int {
 			return cmp.Compare(b.Code, a.Code)
 		})
-	case SortBy == "comment" || SortBy == "comments":
+	case "comment", "comments":
 		slices.SortFunc(summary.Files, func(a, b *FileJob) int {
 			return cmp.Compare(b.Comment, a.Comment)
 		})
-	case SortBy == "complexity" || SortBy == "complexitys" || SortBy == "comp":
+	case "complexity", "complexitys", "comp":
 		slices.SortFunc(summary.Files, func(a, b *FileJob) int {
 			return cmp.Compare(b.Complexity, a.Complexity)
 		})
@@ -244,6 +244,7 @@ func toJSON(input chan *FileJob) string {
 	language := aggregateLanguageSummary(input)
 	language = sortLanguageSummary(language)
 
+	json := jsoniter.ConfigCompatibleWithStandardLibrary
 	jsonString, _ := json.Marshal(language)
 
 	if Debug {
@@ -272,6 +273,7 @@ func toJSON2(input chan *FileJob) string {
 
 	cost, schedule, people := esstimateCostScheduleMonths(sumCode)
 
+	json := jsoniter.ConfigCompatibleWithStandardLibrary
 	jsonString, _ := json.Marshal(Json2{
 		LanguageSummary:         language,
 		EstimatedCost:           cost,
@@ -303,67 +305,18 @@ func toCSVSummary(input chan *FileJob) string {
 	for _, result := range language {
 		records = append(records, []string{
 			result.Name,
-			fmt.Sprint(result.Lines),
-			fmt.Sprint(result.Code),
-			fmt.Sprint(result.Comment),
-			fmt.Sprint(result.Blank),
-			fmt.Sprint(result.Complexity),
-			fmt.Sprint(result.Bytes),
-			fmt.Sprint(result.Count),
-			fmt.Sprint(len(ulocLanguageCount[result.Name])),
+			strconv.FormatInt(result.Lines, 10),
+			strconv.FormatInt(result.Code, 10),
+			strconv.FormatInt(result.Comment, 10),
+			strconv.FormatInt(result.Blank, 10),
+			strconv.FormatInt(result.Complexity, 10),
+			strconv.FormatInt(result.Bytes, 10),
+			strconv.FormatInt(result.Count, 10),
+			strconv.Itoa(len(ulocLanguageCount[result.Name])),
 		})
 	}
 
-	// Cater for the common case of adding plural even for those options that don't make sense
-	// as it's quite common for those who English is not a first language to make a simple mistake
-	switch {
-	case SortBy == "name" || SortBy == "names" || SortBy == "language" || SortBy == "languages":
-		slices.SortFunc(records, func(a, b []string) int {
-			return strings.Compare(a[0], b[0])
-		})
-	case SortBy == "line" || SortBy == "lines":
-		slices.SortFunc(records, func(a, b []string) int {
-			i1, _ := strconv.ParseInt(a[1], 10, 64)
-			i2, _ := strconv.ParseInt(b[1], 10, 64)
-			return cmp.Compare(i2, i1)
-		})
-	case SortBy == "blank" || SortBy == "blanks":
-		slices.SortFunc(records, func(a, b []string) int {
-			i1, _ := strconv.ParseInt(a[4], 10, 64)
-			i2, _ := strconv.ParseInt(b[4], 10, 64)
-			return cmp.Compare(i2, i1)
-		})
-	case SortBy == "code" || SortBy == "codes":
-		slices.SortFunc(records, func(a, b []string) int {
-			i1, _ := strconv.ParseInt(a[2], 10, 64)
-			i2, _ := strconv.ParseInt(b[2], 10, 64)
-			return cmp.Compare(i2, i1)
-		})
-	case SortBy == "comment" || SortBy == "comments":
-		slices.SortFunc(records, func(a, b []string) int {
-			i1, _ := strconv.ParseInt(a[3], 10, 64)
-			i2, _ := strconv.ParseInt(b[3], 10, 64)
-			return cmp.Compare(i2, i1)
-		})
-	case SortBy == "complexity" || SortBy == "complexitys":
-		slices.SortFunc(records, func(a, b []string) int {
-			i1, _ := strconv.ParseInt(a[5], 10, 64)
-			i2, _ := strconv.ParseInt(b[5], 10, 64)
-			return cmp.Compare(i2, i1)
-		})
-	case SortBy == "byte" || SortBy == "bytes":
-		slices.SortFunc(records, func(a, b []string) int {
-			i1, _ := strconv.ParseInt(a[6], 10, 64)
-			i2, _ := strconv.ParseInt(b[6], 10, 64)
-			return cmp.Compare(i2, i1)
-		})
-	default:
-		slices.SortFunc(records, func(a, b []string) int {
-			i1, _ := strconv.ParseInt(a[1], 10, 64)
-			i2, _ := strconv.ParseInt(b[1], 10, 64)
-			return cmp.Compare(i2, i1)
-		})
-	}
+	slices.SortFunc(records, getRecordsSortFunc())
 
 	recordsEnd := [][]string{{
 		"Language",
@@ -387,6 +340,61 @@ func toCSVSummary(input chan *FileJob) string {
 	return b.String()
 }
 
+func getRecordsSortFunc() func(a, b []string) int {
+	// Cater for the common case of adding plural even for those options that don't make sense
+	// as it's quite common for those who English is not a first language to make a simple mistake
+	switch SortBy {
+	case "name", "names":
+		return func(a, b []string) int {
+			return strings.Compare(a[2], b[2])
+		}
+	case "language", "languages", "lang", "langs":
+		return func(a, b []string) int {
+			return strings.Compare(a[0], b[0])
+		}
+	case "line", "lines":
+		return func(a, b []string) int {
+			i1, _ := strconv.ParseInt(a[3], 10, 64)
+			i2, _ := strconv.ParseInt(b[3], 10, 64)
+			return cmp.Compare(i2, i1)
+		}
+	case "blank", "blanks":
+		return func(a, b []string) int {
+			i1, _ := strconv.ParseInt(a[6], 10, 64)
+			i2, _ := strconv.ParseInt(b[6], 10, 64)
+			return cmp.Compare(i2, i1)
+		}
+	case "code", "codes":
+		return func(a, b []string) int {
+			i1, _ := strconv.ParseInt(a[4], 10, 64)
+			i2, _ := strconv.ParseInt(b[4], 10, 64)
+			return cmp.Compare(i2, i1)
+		}
+	case "comment", "comments":
+		return func(a, b []string) int {
+			i1, _ := strconv.ParseInt(a[5], 10, 64)
+			i2, _ := strconv.ParseInt(b[5], 10, 64)
+			return cmp.Compare(i2, i1)
+		}
+	case "complexity", "complexitys":
+		return func(a, b []string) int {
+			i1, _ := strconv.ParseInt(a[7], 10, 64)
+			i2, _ := strconv.ParseInt(b[7], 10, 64)
+			return cmp.Compare(i2, i1)
+		}
+	case "byte", "bytes":
+		return func(a, b []string) int {
+			i1, _ := strconv.ParseInt(a[8], 10, 64)
+			i2, _ := strconv.ParseInt(b[8], 10, 64)
+			return cmp.Compare(i2, i1)
+		}
+	default:
+		return func(a, b []string) int {
+			return strings.Compare(a[2], b[2])
+		}
+	}
+}
+
 func toCSVFiles(input chan *FileJob) string {
 	records := [][]string{}
 
@@ -395,68 +403,17 @@ func toCSVFiles(input chan *FileJob) string {
 			result.Language,
 			result.Location,
 			result.Filename,
-			fmt.Sprint(result.Lines),
-			fmt.Sprint(result.Code),
-			fmt.Sprint(result.Comment),
-			fmt.Sprint(result.Blank),
-			fmt.Sprint(result.Complexity),
-			fmt.Sprint(result.Bytes),
-			fmt.Sprint(result.Uloc),
+			strconv.FormatInt(result.Lines, 10),
+			strconv.FormatInt(result.Code, 10),
+			strconv.FormatInt(result.Comment, 10),
+			strconv.FormatInt(result.Blank, 10),
+			strconv.FormatInt(result.Complexity, 10),
+			strconv.FormatInt(result.Bytes, 10),
+			strconv.Itoa(result.Uloc),
 		})
 	}
 
-	// Cater for the common case of adding plural even for those options that don't make sense
-	// as it's quite common for those who English is not a first language to make a simple mistake
-	switch {
-	case SortBy == "name" || SortBy == "names":
-		slices.SortFunc(records, func(a, b []string) int {
-			return strings.Compare(a[2], b[2])
-		})
-	case SortBy == "language" || SortBy == "languages":
-		slices.SortFunc(records, func(a, b []string) int {
-			return strings.Compare(a[0], b[0])
-		})
-	case SortBy == "line" || SortBy == "lines":
-		slices.SortFunc(records, func(a, b []string) int {
-			i1, _ := strconv.ParseInt(a[3], 10, 64)
-			i2, _ := strconv.ParseInt(b[3], 10, 64)
-			return cmp.Compare(i2, i1)
-		})
-	case SortBy == "blank" || SortBy == "blanks":
-		slices.SortFunc(records, func(a, b []string) int {
-			i1, _ := strconv.ParseInt(a[6], 10, 64)
-			i2, _ := strconv.ParseInt(b[6], 10, 64)
-			return cmp.Compare(i2, i1)
-		})
-	case SortBy == "code" || SortBy == "codes":
-		slices.SortFunc(records, func(a, b []string) int {
-			i1, _ := strconv.ParseInt(a[4], 10, 64)
-			i2, _ := strconv.ParseInt(b[4], 10, 64)
-			return cmp.Compare(i2, i1)
-		})
-	case SortBy == "comment" || SortBy == "comments":
-		slices.SortFunc(records, func(a, b []string) int {
-			i1, _ := strconv.ParseInt(a[5], 10, 64)
-			i2, _ := strconv.ParseInt(b[5], 10, 64)
-			return cmp.Compare(i2, i1)
-		})
-	case SortBy == "complexity" || SortBy == "complexitys":
-		slices.SortFunc(records, func(a, b []string) int {
-			i1, _ := strconv.ParseInt(a[7], 10, 64)
-			i2, _ := strconv.ParseInt(b[7], 10, 64)
-			return cmp.Compare(i2, i1)
-		})
-	case SortBy == "byte" || SortBy == "bytes":
-		slices.SortFunc(records, func(a, b []string) int {
-			i1, _ := strconv.ParseInt(a[8], 10, 64)
-			i2, _ := strconv.ParseInt(b[8], 10, 64)
-			return cmp.Compare(i2, i1)
-		})
-	default:
-		slices.SortFunc(records, func(a, b []string) int {
-			return strings.Compare(a[2], b[2])
-		})
-	}
+	slices.SortFunc(records, getRecordsSortFunc())
 
 	recordsEnd := [][]string{{
 		"Language",
@@ -519,7 +476,7 @@ func toOpenMetricsFiles(input chan *FileJob) string {
 		fmt.Fprintf(sb, openMetricsFileRecordFormat, "complexity", file.Language, filename, file.Complexity)
 		fmt.Fprintf(sb, openMetricsFileRecordFormat, "bytes", file.Language, filename, file.Bytes)
 	}
-	sb.WriteString("# EOF")
+	sb.WriteString("# EOF\n")
 	return sb.String()
 }
 
@@ -536,17 +493,17 @@ func toCSVStream(input chan *FileJob) string {
 		var location = "\"" + quoteRegex.ReplaceAllString(result.Location, "\"\"") + "\""
 		var filename = "\"" + quoteRegex.ReplaceAllString(result.Filename, "\"\"") + "\""
 
-		fmt.Printf("%s,%s,%s,%s,%s,%s,%s,%s,%s,%s\n",
+		fmt.Printf("%s,%s,%s,%d,%d,%d,%d,%d,%d,%d\n",
 			result.Language,
 			location,
 			filename,
-			fmt.Sprint(result.Lines),
-			fmt.Sprint(result.Code),
-			fmt.Sprint(result.Comment),
-			fmt.Sprint(result.Blank),
-			fmt.Sprint(result.Complexity),
-			fmt.Sprint(result.Bytes),
-			fmt.Sprint(result.Uloc),
+			result.Lines,
+			result.Code,
+			result.Comment,
+			result.Blank,
+			result.Complexity,
+			result.Bytes,
+			result.Uloc,
 		)
 	}
 
@@ -556,7 +513,7 @@ func toCSVStream(input chan *FileJob) string {
 func toHtml(input chan *FileJob) string {
 	return `<html lang="en"><head><meta charset="utf-8" /><title>scc html output</title><style>table { border-collapse: collapse; }td, th { border: 1px solid #999; padding: 0.5rem; text-align: left;}</style></head><body>` +
 		toHtmlTable(input) +
-		`</body></html>`
+		"</body></html>\n"
 }
 
 func toHtmlTable(input chan *FileJob) string {
@@ -655,7 +612,7 @@ func toHtmlTable(input chan *FileJob) string {
 		<td>%d</td>
 		<td>%d</td>
 		<td>%d</td>
-	    <td>%d</td>
+		<td>%d</td>
 		<td>%d</td>
 	</tr>`, res.Location, res.Lines, res.Blank, res.Comment, res.Code, res.Complexity, res.Bytes, res.Uloc)
 			}
@@ -672,7 +629,7 @@ func toHtmlTable(input chan *FileJob) string {
 		<th>%d</th>
 		<th>%d</th>
 		<th>%d</th>
-    	<th>%d</th>
+		<th>%d</th>
 		<th>%d</th>
 	</tr>`, sumFiles, sumLines, sumBlank, sumComment, sumCode, sumComplexity, sumBytes, len(ulocGlobalCount))
 
@@ -833,7 +790,7 @@ func fileSummarizeMulti(input chan *FileJob) string {
 	var str strings.Builder
 
 	// for each output pump the results into
-	for _, s := range strings.Split(FormatMulti, ",") {
+	for s := range strings.SplitSeq(FormatMulti, ",") {
 		t := strings.Split(s, ":")
 		if len(t) == 2 {
 			i := make(chan *FileJob, len(results))
@@ -964,38 +921,7 @@ func fileSummarizeLong(input chan *FileJob) string {
 		language = append(language, summary)
 	}
 
-	// Cater for the common case of adding plural even for those options that don't make sense
-	// as it's quite common for those who English is not a first language to make a simple mistake
-	switch {
-	case SortBy == "name" || SortBy == "names" || SortBy == "language" || SortBy == "langs":
-		slices.SortFunc(language, func(a, b LanguageSummary) int {
-			return strings.Compare(a.Name, b.Name)
-		})
-	case SortBy == "line" || SortBy == "lines":
-		slices.SortFunc(language, func(a, b LanguageSummary) int {
-			return cmp.Compare(b.Lines, a.Lines)
-		})
-	case SortBy == "blank" || SortBy == "blanks":
-		slices.SortFunc(language, func(a, b LanguageSummary) int {
-			return cmp.Compare(b.Blank, a.Blank)
-		})
-	case SortBy == "code" || SortBy == "codes":
-		slices.SortFunc(language, func(a, b LanguageSummary) int {
-			return cmp.Compare(b.Code, a.Code)
-		})
-	case SortBy == "comment" || SortBy == "comments":
-		slices.SortFunc(language, func(a, b LanguageSummary) int {
-			return cmp.Compare(b.Comment, a.Comment)
-		})
-	case SortBy == "complexity" || SortBy == "complexitys":
-		slices.SortFunc(language, func(a, b LanguageSummary) int {
-			return cmp.Compare(b.Complexity, a.Complexity)
-		})
-	default:
-		slices.SortFunc(language, func(a, b LanguageSummary) int {
-			return cmp.Compare(b.Count, a.Count)
-		})
-	}
+	language = sortLanguageSummary(language)
 
 	startTime := makeTimestampMilli()
 	for _, summary := range language {
@@ -1308,14 +1234,7 @@ func maxIn(i []int) int {
 		return 0
 	}
 
-	m := i[0]
-	for _, x := range i {
-		if x > m {
-			m = x
-		}
-	}
-
-	return m
+	return slices.Max(i)
 }
 
 func meanIn(i []int) int {
@@ -1427,16 +1346,10 @@ func isLeapYear(year int) bool {
 	leapFlag := false
 	if year%4 == 0 {
 		if year%100 == 0 {
-			if year%400 == 0 {
-				leapFlag = true
-			} else {
-				leapFlag = false
-			}
+			leapFlag = year%400 == 0
 		} else {
 			leapFlag = true
 		}
-	} else {
-		leapFlag = false
 	}
 	return leapFlag
 }
@@ -1501,40 +1414,40 @@ func sortLanguageSummary(language []LanguageSummary) []LanguageSummary {
 	// as it's quite common for those who English is not a first language to make a simple mistake
 	// NB in any non name cases if the values are the same we sort by name to ensure
 	// deterministic output
-	switch {
-	case SortBy == "name" || SortBy == "names" || SortBy == "language" || SortBy == "languages":
+	switch SortBy {
+	case "name", "names", "language", "languages", "lang", "langs":
 		slices.SortFunc(language, func(a, b LanguageSummary) int {
 			return strings.Compare(a.Name, b.Name)
 		})
-	case SortBy == "line" || SortBy == "lines":
+	case "line", "lines":
 		slices.SortFunc(language, func(a, b LanguageSummary) int {
 			if order := cmp.Compare(b.Lines, a.Lines); order != 0 {
 				return order
 			}
 			return strings.Compare(a.Name, b.Name)
 		})
-	case SortBy == "blank" || SortBy == "blanks":
+	case "blank", "blanks":
 		slices.SortFunc(language, func(a, b LanguageSummary) int {
 			if order := cmp.Compare(b.Blank, a.Blank); order != 0 {
 				return order
 			}
 			return strings.Compare(a.Name, b.Name)
 		})
-	case SortBy == "code" || SortBy == "codes":
+	case "code", "codes":
 		slices.SortFunc(language, func(a, b LanguageSummary) int {
 			if order := cmp.Compare(b.Code, a.Code); order != 0 {
 				return order
 			}
 			return strings.Compare(a.Name, b.Name)
 		})
-	case SortBy == "comment" || SortBy == "comments":
+	case "comment", "comments":
 		slices.SortFunc(language, func(a, b LanguageSummary) int {
 			if order := cmp.Compare(b.Comment, a.Comment); order != 0 {
 				return order
 			}
 			return strings.Compare(a.Name, b.Name)
 		})
-	case SortBy == "complexity" || SortBy == "complexitys":
+	case "complexity", "complexitys":
 		slices.SortFunc(language, func(a, b LanguageSummary) int {
 			if order := cmp.Compare(b.Complexity, a.Complexity); order != 0 {
 				return order
